# Generated by roxygen2: do not edit by hand

S3method(coda::as.mcmc.list,ShrinkageTrees)
S3method(plot,CausalShrinkageForest)
S3method(plot,ShrinkageTrees)
S3method(plot,ShrinkageTreesPrediction)
S3method(predict,CausalShrinkageForest)
S3method(predict,ShrinkageTrees)
S3method(print,CausalShrinkageForest)
S3method(print,CausalShrinkageForestPrediction)
S3method(print,ShrinkageTrees)
S3method(print,ShrinkageTreesPrediction)
S3method(print,summary.CausalShrinkageForest)
S3method(print,summary.CausalShrinkageForestPrediction)
S3method(print,summary.ShrinkageTrees)
S3method(print,summary.ShrinkageTreesPrediction)
S3method(summary,CausalShrinkageForest)
S3method(summary,CausalShrinkageForestPrediction)
S3method(summary,ShrinkageTrees)
S3method(summary,ShrinkageTreesPrediction)
export(CausalHorseForest)
export(CausalShrinkageForest)
export(HorseTrees)
export(ShrinkageTrees)
export(SurvivalBART)
export(SurvivalBCF)
export(SurvivalDART)
export(SurvivalShrinkageBCF)
export(bayesian_bootstrap_ate)
importFrom(Rcpp,evalCpp)
importFrom(parallel,detectCores)
importFrom(parallel,mclapply)
importFrom(stats,dnorm)
importFrom(stats,median)
importFrom(stats,pnorm)
importFrom(stats,qchisq)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rexp)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(utils,head)
useDynLib(ShrinkageTrees, .registration = TRUE)
