#' @keywords internal
#' @importFrom stats median quantile pnorm sd
#' @importFrom utils head
"_PACKAGE"

# Suppress R CMD check NOTE for .data used inside ggplot2 aes() calls.
utils::globalVariables(".data")

