# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

CausalHorseForest_cpp <- function(nSEXP, p_treatSEXP, p_controlSEXP, X_train_treatSEXP, X_train_controlSEXP, ySEXP, status_indicatorSEXP, is_survivalSEXP, observed_left_timeSEXP, observed_right_timeSEXP, interval_censoring_indicatorSEXP, treatment_indicatorSEXP, n_testSEXP, X_test_controlSEXP, X_test_treatSEXP, treatment_indicator_testSEXP, no_trees_treatSEXP, power_treatSEXP, base_treatSEXP, p_grow_treatSEXP, p_prune_treatSEXP, omega_treatSEXP, prior_type_treatSEXP, param1_treatSEXP, param2_treatSEXP, reversible_treatSEXP, dirichlet_bool_treatSEXP, a_dirichlet_treatSEXP, b_dirichlet_treatSEXP, rho_dirichlet_treatSEXP, no_trees_controlSEXP, power_controlSEXP, base_controlSEXP, p_grow_controlSEXP, p_prune_controlSEXP, omega_controlSEXP, prior_type_controlSEXP, param1_controlSEXP, param2_controlSEXP, reversible_controlSEXP, dirichlet_bool_controlSEXP, a_dirichlet_controlSEXP, b_dirichlet_controlSEXP, rho_dirichlet_controlSEXP, sigma_knownSEXP, sigmaSEXP, lambdaSEXP, nuSEXP, N_postSEXP, N_burnSEXP, delayed_proposalSEXP, store_posterior_sample_controlSEXP, store_posterior_sample_treatSEXP, verboseSEXP, treatment_codingSEXP, propensity_trainSEXP, propensity_testSEXP) {
    .Call(`_ShrinkageTrees_CausalHorseForest_cpp`, nSEXP, p_treatSEXP, p_controlSEXP, X_train_treatSEXP, X_train_controlSEXP, ySEXP, status_indicatorSEXP, is_survivalSEXP, observed_left_timeSEXP, observed_right_timeSEXP, interval_censoring_indicatorSEXP, treatment_indicatorSEXP, n_testSEXP, X_test_controlSEXP, X_test_treatSEXP, treatment_indicator_testSEXP, no_trees_treatSEXP, power_treatSEXP, base_treatSEXP, p_grow_treatSEXP, p_prune_treatSEXP, omega_treatSEXP, prior_type_treatSEXP, param1_treatSEXP, param2_treatSEXP, reversible_treatSEXP, dirichlet_bool_treatSEXP, a_dirichlet_treatSEXP, b_dirichlet_treatSEXP, rho_dirichlet_treatSEXP, no_trees_controlSEXP, power_controlSEXP, base_controlSEXP, p_grow_controlSEXP, p_prune_controlSEXP, omega_controlSEXP, prior_type_controlSEXP, param1_controlSEXP, param2_controlSEXP, reversible_controlSEXP, dirichlet_bool_controlSEXP, a_dirichlet_controlSEXP, b_dirichlet_controlSEXP, rho_dirichlet_controlSEXP, sigma_knownSEXP, sigmaSEXP, lambdaSEXP, nuSEXP, N_postSEXP, N_burnSEXP, delayed_proposalSEXP, store_posterior_sample_controlSEXP, store_posterior_sample_treatSEXP, verboseSEXP, treatment_codingSEXP, propensity_trainSEXP, propensity_testSEXP)
}

HorseTrees_cpp <- function(nSEXP, pSEXP, n_testSEXP, X_trainSEXP, ySEXP, status_indicatorSEXP, is_survivalSEXP, observed_left_timeSEXP, observed_right_timeSEXP, interval_censoring_indicatorSEXP, X_testSEXP, number_of_treesSEXP, N_postSEXP, N_burnSEXP, delayed_proposalSEXP, powerSEXP, baseSEXP, p_growSEXP, p_pruneSEXP, nuSEXP, lambdaSEXP, dirichlet_boolSEXP, a_dirichletSEXP, b_dirichletSEXP, rho_dirichletSEXP, sigmaSEXP, sigma_knownSEXP, omegaSEXP, param1SEXP, param2SEXP, prior_typeSEXP, reversibleSEXP, store_parametersSEXP, store_posterior_sampleSEXP, verboseSEXP) {
    .Call(`_ShrinkageTrees_HorseTrees_cpp`, nSEXP, pSEXP, n_testSEXP, X_trainSEXP, ySEXP, status_indicatorSEXP, is_survivalSEXP, observed_left_timeSEXP, observed_right_timeSEXP, interval_censoring_indicatorSEXP, X_testSEXP, number_of_treesSEXP, N_postSEXP, N_burnSEXP, delayed_proposalSEXP, powerSEXP, baseSEXP, p_growSEXP, p_pruneSEXP, nuSEXP, lambdaSEXP, dirichlet_boolSEXP, a_dirichletSEXP, b_dirichletSEXP, rho_dirichletSEXP, sigmaSEXP, sigma_knownSEXP, omegaSEXP, param1SEXP, param2SEXP, prior_typeSEXP, reversibleSEXP, store_parametersSEXP, store_posterior_sampleSEXP, verboseSEXP)
}

probitHorseTrees_cpp <- function(nSEXP, pSEXP, n_testSEXP, X_trainSEXP, ySEXP, X_testSEXP, number_of_treesSEXP, N_postSEXP, N_burnSEXP, delayed_proposalSEXP, powerSEXP, baseSEXP, p_growSEXP, p_pruneSEXP, omegaSEXP, latent_thresholdSEXP, param1SEXP, param2SEXP, prior_typeSEXP, reversibleSEXP, store_posterior_sampleSEXP, verboseSEXP) {
    .Call(`_ShrinkageTrees_probitHorseTrees_cpp`, nSEXP, pSEXP, n_testSEXP, X_trainSEXP, ySEXP, X_testSEXP, number_of_treesSEXP, N_postSEXP, N_burnSEXP, delayed_proposalSEXP, powerSEXP, baseSEXP, p_growSEXP, p_pruneSEXP, omegaSEXP, latent_thresholdSEXP, param1SEXP, param2SEXP, prior_typeSEXP, reversibleSEXP, store_posterior_sampleSEXP, verboseSEXP)
}

