% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.ShrinkageTrees}
\alias{print.ShrinkageTrees}
\title{Print a ShrinkageTrees model}
\usage{
\method{print}{ShrinkageTrees}(x, ...)
}
\arguments{
\item{x}{A fitted \code{ShrinkageTrees} model object.}

\item{...}{Currently unused.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays a concise summary of a fitted \code{ShrinkageTrees} model,
including outcome type, prior, MCMC settings, acceptance ratio, and
posterior mean sigma.
}
\seealso{
\code{\link{summary.ShrinkageTrees}}, \code{\link{HorseTrees}},
\code{\link{ShrinkageTrees}}
}
