% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.CausalShrinkageForest}
\alias{print.CausalShrinkageForest}
\title{Print a CausalShrinkageForest model}
\usage{
\method{print}{CausalShrinkageForest}(x, ...)
}
\arguments{
\item{x}{A fitted \code{CausalShrinkageForest} model object.}

\item{...}{Currently unused.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays a concise summary of a fitted \code{CausalShrinkageForest} model
with per-forest columns for priors, tree counts, feature counts, and MCMC
acceptance ratios.
}
\seealso{
\code{\link{summary.CausalShrinkageForest}},
\code{\link{CausalHorseForest}}, \code{\link{CausalShrinkageForest}}
}
