% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{summary.ShrinkageTreesPrediction}
\alias{summary.ShrinkageTreesPrediction}
\title{Summarise a ShrinkageTreesPrediction object}
\usage{
\method{summary}{ShrinkageTreesPrediction}(object, ...)
}
\arguments{
\item{object}{A \code{ShrinkageTreesPrediction} object.}

\item{...}{Currently unused.}
}
\value{
A \code{summary.ShrinkageTreesPrediction} object.
}
\description{
Returns distributional summaries (min, Q1, median, max) of the posterior
mean predictions and credible interval bounds across all observations.
}
\seealso{
\code{\link{predict.ShrinkageTrees}},
\code{\link{print.summary.ShrinkageTreesPrediction}}
}
