% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.summary.CausalShrinkageForest}
\alias{print.summary.CausalShrinkageForest}
\title{Print a CausalShrinkageForest model summary}
\usage{
\method{print}{summary.CausalShrinkageForest}(x, n_vi = 10, ...)
}
\arguments{
\item{x}{A \code{summary.CausalShrinkageForest} object.}

\item{n_vi}{Maximum number of variables to display per variable importance
table. Default \code{10}.}

\item{...}{Currently unused.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays a detailed summary of a \code{CausalShrinkageForest} model,
including model specification, treatment effect estimates, prognostic
function, posterior sigma, variable importance for each forest, and MCMC
diagnostics.
}
\seealso{
\code{\link{summary.CausalShrinkageForest}}
}
