% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.summary.ShrinkageTreesPrediction}
\alias{print.summary.ShrinkageTreesPrediction}
\title{Print a ShrinkageTreesPrediction summary}
\usage{
\method{print}{summary.ShrinkageTreesPrediction}(x, digits = 3, ...)
}
\arguments{
\item{x}{A \code{summary.ShrinkageTreesPrediction} object.}

\item{digits}{Number of decimal places. Default \code{3}.}

\item{...}{Currently unused.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays distributional summaries (min, Q1, median, max) of the posterior
mean predictions and credible interval bounds.
}
\seealso{
\code{\link{summary.ShrinkageTreesPrediction}}
}
